# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA = Metadata(
    id="e37a5b8e4d1b3bc448995c7e8c944e1559bba40f.boutiques",
    name="antsApplyTransformsToPoints",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsApplyTransformsToPointsSingleTransformParamsDictNoTag = typing.TypedDict('_AntsApplyTransformsToPointsSingleTransformParamsDictNoTag', {})
AntsApplyTransformsToPointsSingleTransformParamsDictTagged = typing.TypedDict('AntsApplyTransformsToPointsSingleTransformParamsDictTagged', {
    "@type": typing.Literal["single_transform"],
})
AntsApplyTransformsToPointsSingleTransformParamsDict = _AntsApplyTransformsToPointsSingleTransformParamsDictNoTag | AntsApplyTransformsToPointsSingleTransformParamsDictTagged


_AntsApplyTransformsToPointsInverseTransformParamsDictNoTag = typing.TypedDict('_AntsApplyTransformsToPointsInverseTransformParamsDictNoTag', {
    "transform_file": InputPathType,
})
AntsApplyTransformsToPointsInverseTransformParamsDictTagged = typing.TypedDict('AntsApplyTransformsToPointsInverseTransformParamsDictTagged', {
    "@type": typing.Literal["inverse_transform"],
    "transform_file": InputPathType,
})
AntsApplyTransformsToPointsInverseTransformParamsDict = _AntsApplyTransformsToPointsInverseTransformParamsDictNoTag | AntsApplyTransformsToPointsInverseTransformParamsDictTagged


_AntsApplyTransformsToPointsParamsDictNoTag = typing.TypedDict('_AntsApplyTransformsToPointsParamsDictNoTag', {
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "precision": typing.NotRequired[bool | None],
    "forantsr": typing.NotRequired[bool | None],
    "input": InputPathType,
    "output": str,
    "transform": typing.NotRequired[typing.Union[AntsApplyTransformsToPointsSingleTransformParamsDictTagged, AntsApplyTransformsToPointsInverseTransformParamsDictTagged] | None],
})
AntsApplyTransformsToPointsParamsDictTagged = typing.TypedDict('AntsApplyTransformsToPointsParamsDictTagged', {
    "@type": typing.Literal["ants/antsApplyTransformsToPoints"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "precision": typing.NotRequired[bool | None],
    "forantsr": typing.NotRequired[bool | None],
    "input": InputPathType,
    "output": str,
    "transform": typing.NotRequired[typing.Union[AntsApplyTransformsToPointsSingleTransformParamsDictTagged, AntsApplyTransformsToPointsInverseTransformParamsDictTagged] | None],
})
AntsApplyTransformsToPointsParamsDict = _AntsApplyTransformsToPointsParamsDictNoTag | AntsApplyTransformsToPointsParamsDictTagged


def ants_apply_transforms_to_points_transform_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "single_transform": ants_apply_transforms_to_points_single_transform_cargs,
        "inverse_transform": ants_apply_transforms_to_points_inverse_transform_cargs,
    }.get(t)


def ants_apply_transforms_to_points_transform_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "single_transform": ants_apply_transforms_to_points_single_transform_validate,
        "inverse_transform": ants_apply_transforms_to_points_inverse_transform_validate,
    }.get(t)


def ants_apply_transforms_to_points_single_transform(
) -> AntsApplyTransformsToPointsSingleTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "single_transform",
    }
    return params


def ants_apply_transforms_to_points_single_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsApplyTransformsToPointsSingleTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')


def ants_apply_transforms_to_points_single_transform_cargs(
    params: AntsApplyTransformsToPointsSingleTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[TRANSFORM]")
    return cargs


def ants_apply_transforms_to_points_inverse_transform(
    transform_file: InputPathType,
) -> AntsApplyTransformsToPointsInverseTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        transform_file:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "inverse_transform",
        "transform_file": transform_file,
    }
    return params


def ants_apply_transforms_to_points_inverse_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsApplyTransformsToPointsInverseTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("transform_file", None) is None:
        raise StyxValidationError("`transform_file` must not be None")
    if not isinstance(params["transform_file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`transform_file` has the wrong type: Received `{type(params.get("transform_file", None))}` expected `InputPathType`')


def ants_apply_transforms_to_points_inverse_transform_cargs(
    params: AntsApplyTransformsToPointsInverseTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("transform_file", None)) + ",1")
    return cargs


class AntsApplyTransformsToPointsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsApplyTransformsToPointsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    warped_points: OutputPathType
    """The output is the CSV file containing warped points."""


def ants_apply_transforms_to_points_params(
    input_: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    precision: bool | None = None,
    forantsr: bool | None = None,
    transform: typing.Union[AntsApplyTransformsToPointsSingleTransformParamsDictTagged, AntsApplyTransformsToPointsInverseTransformParamsDictTagged] | None = None,
) -> AntsApplyTransformsToPointsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: Input CSV file with columns including x,y,z,t headers that\
            define the points in physical space, or a 2D .mha binary image file.
        output: Output the warped points to a CSV file.
        dimensionality: This option forces the points to be treated as a\
            specified-dimensionality.
        precision: Use double precision.
        forantsr: Set true for ANTsR IO.
        transform: Transform file(s) to apply to the input points. Uses an\
            inverse transform if specified as [transformFileName,1].
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsApplyTransformsToPoints",
        "input": input_,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if precision is not None:
        params["precision"] = precision
    if forantsr is not None:
        params["forantsr"] = forantsr
    if transform is not None:
        params["transform"] = transform
    return params


def ants_apply_transforms_to_points_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsApplyTransformsToPointsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensionality", None) is not None:
        if not isinstance(params["dimensionality"], int):
            raise StyxValidationError(f'`dimensionality` has the wrong type: Received `{type(params.get("dimensionality", None))}` expected `typing.Literal[2, 3] | None`')
        if params["dimensionality"] not in [2, 3]:
            raise StyxValidationError("Parameter `dimensionality` must be one of [2, 3]")
    if params.get("precision", None) is not None:
        if not isinstance(params["precision"], bool):
            raise StyxValidationError(f'`precision` has the wrong type: Received `{type(params.get("precision", None))}` expected `bool | None`')
    if params.get("forantsr", None) is not None:
        if not isinstance(params["forantsr"], bool):
            raise StyxValidationError(f'`forantsr` has the wrong type: Received `{type(params.get("forantsr", None))}` expected `bool | None`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("transform", None) is not None:
        if not isinstance(params["transform"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["transform"])}\'')
        if "@type" not in params["transform"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["transform"]["@type"] not in ["single_transform", "inverse_transform"]:
            raise StyxValidationError("Parameter `transform`s `@type` must be one of [\"single_transform\", \"inverse_transform\"]")
        ants_apply_transforms_to_points_transform_validate_dyn_fn(params["transform"]["@type"])(params["transform"])


def ants_apply_transforms_to_points_cargs(
    params: AntsApplyTransformsToPointsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsApplyTransformsToPoints")
    if params.get("dimensionality", None) is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality", None))
        ])
    if params.get("precision", None) is not None:
        cargs.extend([
            "--precision",
            ("1" if params.get("precision", None) else "0")
        ])
    if params.get("forantsr", None) is not None:
        cargs.extend([
            "--forantsr",
            ("1" if params.get("forantsr", None) else "0")
        ])
    cargs.extend([
        "-i",
        execution.input_file(params.get("input", None))
    ])
    cargs.extend([
        "-o",
        params.get("output", None)
    ])
    if params.get("transform", None) is not None:
        cargs.extend([
            "-t",
            *ants_apply_transforms_to_points_transform_cargs_dyn_fn(params.get("transform", None)["@type"])(params.get("transform", None), execution)
        ])
    return cargs


def ants_apply_transforms_to_points_outputs(
    params: AntsApplyTransformsToPointsParamsDict,
    execution: Execution,
) -> AntsApplyTransformsToPointsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsToPointsOutputs(
        root=execution.output_file("."),
        warped_points=execution.output_file(params.get("output", None)),
    )
    return ret


def ants_apply_transforms_to_points_execute(
    params: AntsApplyTransformsToPointsParamsDict,
    runner: Runner | None = None,
) -> AntsApplyTransformsToPointsOutputs:
    """
    antsApplyTransformsToPoints
    
    antsApplyTransformsToPoints, applied to an input image, transforms it
    according to a reference image and a transform (or a set of transforms). It
    reads in a CSV file with the first D columns defining the spatial location
    where the spatial location is defined in physical coordinates. The CSV file
    should have a header row.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsToPointsOutputs`).
    """
    ants_apply_transforms_to_points_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA)
    params = execution.params(params)
    cargs = ants_apply_transforms_to_points_cargs(params, execution)
    ret = ants_apply_transforms_to_points_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_apply_transforms_to_points(
    input_: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    precision: bool | None = None,
    forantsr: bool | None = None,
    transform: typing.Union[AntsApplyTransformsToPointsSingleTransformParamsDictTagged, AntsApplyTransformsToPointsInverseTransformParamsDictTagged] | None = None,
    runner: Runner | None = None,
) -> AntsApplyTransformsToPointsOutputs:
    """
    antsApplyTransformsToPoints
    
    antsApplyTransformsToPoints, applied to an input image, transforms it
    according to a reference image and a transform (or a set of transforms). It
    reads in a CSV file with the first D columns defining the spatial location
    where the spatial location is defined in physical coordinates. The CSV file
    should have a header row.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_: Input CSV file with columns including x,y,z,t headers that\
            define the points in physical space, or a 2D .mha binary image file.
        output: Output the warped points to a CSV file.
        dimensionality: This option forces the points to be treated as a\
            specified-dimensionality.
        precision: Use double precision.
        forantsr: Set true for ANTsR IO.
        transform: Transform file(s) to apply to the input points. Uses an\
            inverse transform if specified as [transformFileName,1].
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsToPointsOutputs`).
    """
    params = ants_apply_transforms_to_points_params(
        dimensionality=dimensionality,
        precision=precision,
        forantsr=forantsr,
        input_=input_,
        output=output,
        transform=transform,
    )
    return ants_apply_transforms_to_points_execute(params, runner)


__all__ = [
    "ANTS_APPLY_TRANSFORMS_TO_POINTS_METADATA",
    "AntsApplyTransformsToPointsInverseTransformParamsDict",
    "AntsApplyTransformsToPointsInverseTransformParamsDictTagged",
    "AntsApplyTransformsToPointsOutputs",
    "AntsApplyTransformsToPointsParamsDict",
    "AntsApplyTransformsToPointsParamsDictTagged",
    "AntsApplyTransformsToPointsSingleTransformParamsDict",
    "AntsApplyTransformsToPointsSingleTransformParamsDictTagged",
    "ants_apply_transforms_to_points",
    "ants_apply_transforms_to_points_execute",
    "ants_apply_transforms_to_points_inverse_transform",
    "ants_apply_transforms_to_points_params",
    "ants_apply_transforms_to_points_single_transform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_JOINT_LABEL_FUSION_SH_METADATA = Metadata(
    id="c1267bf8b624408f46319d6d24c21b8a6d4c38ca.boutiques",
    name="antsJointLabelFusion.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsJointLabelFusionShParamsDictNoTag = typing.TypedDict('_AntsJointLabelFusionShParamsDictNoTag', {
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "output": typing.NotRequired[str | None],
    "atlas_image_mrf": typing.NotRequired[str | None],
    "atlas_segmentation_mrf": typing.NotRequired[str | None],
    "rigid_transform": typing.NotRequired[str | None],
    "similarity_metric": typing.NotRequired[str | None],
    "other_options": typing.NotRequired[str | None],
    "verbose": typing.NotRequired[bool | None],
    "target_image": str,
    "mask_image": str,
    "rigid_transform_additional_options": typing.NotRequired[str | None],
    "similarity_metric_additional_options": typing.NotRequired[str | None],
})
AntsJointLabelFusionShParamsDictTagged = typing.TypedDict('AntsJointLabelFusionShParamsDictTagged', {
    "@type": typing.Literal["ants/antsJointLabelFusion.sh"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "output": typing.NotRequired[str | None],
    "atlas_image_mrf": typing.NotRequired[str | None],
    "atlas_segmentation_mrf": typing.NotRequired[str | None],
    "rigid_transform": typing.NotRequired[str | None],
    "similarity_metric": typing.NotRequired[str | None],
    "other_options": typing.NotRequired[str | None],
    "verbose": typing.NotRequired[bool | None],
    "target_image": str,
    "mask_image": str,
    "rigid_transform_additional_options": typing.NotRequired[str | None],
    "similarity_metric_additional_options": typing.NotRequired[str | None],
})
AntsJointLabelFusionShParamsDict = _AntsJointLabelFusionShParamsDictNoTag | AntsJointLabelFusionShParamsDictTagged


class AntsJointLabelFusionShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsJointLabelFusionShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    segmentation_output: OutputPathType | None
    """Output segmented image."""


def ants_joint_label_fusion_sh_params(
    target_image: str,
    mask_image: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    output: str | None = None,
    atlas_image_mrf: str | None = None,
    atlas_segmentation_mrf: str | None = None,
    rigid_transform: str | None = None,
    similarity_metric: str | None = None,
    other_options: str | None = None,
    verbose: bool | None = None,
    rigid_transform_additional_options: str | None = None,
    similarity_metric_additional_options: str | None = None,
) -> AntsJointLabelFusionShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        target_image: Image to segment.
        mask_image: Mask image.
        dimensionality: Image dimensionality (2 or 3).
        output: Root directory for the output segmentation.
        atlas_image_mrf: Atlas image(s) to be used for MRF initialization.
        atlas_segmentation_mrf: Atlas segmentation(s) to be used for MRF\
            initialization.
        rigid_transform: Rigid transform initialization.
        similarity_metric: Metric used for calculating similarity.
        other_options: Additional options for label fusion.
        verbose: Verbose output.
        rigid_transform_additional_options: Additional options for rigid\
            transform.
        similarity_metric_additional_options: Additional options for similarity\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsJointLabelFusion.sh",
        "target_image": target_image,
        "mask_image": mask_image,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if output is not None:
        params["output"] = output
    if atlas_image_mrf is not None:
        params["atlas_image_mrf"] = atlas_image_mrf
    if atlas_segmentation_mrf is not None:
        params["atlas_segmentation_mrf"] = atlas_segmentation_mrf
    if rigid_transform is not None:
        params["rigid_transform"] = rigid_transform
    if similarity_metric is not None:
        params["similarity_metric"] = similarity_metric
    if other_options is not None:
        params["other_options"] = other_options
    if verbose is not None:
        params["verbose"] = verbose
    if rigid_transform_additional_options is not None:
        params["rigid_transform_additional_options"] = rigid_transform_additional_options
    if similarity_metric_additional_options is not None:
        params["similarity_metric_additional_options"] = similarity_metric_additional_options
    return params


def ants_joint_label_fusion_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsJointLabelFusionShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensionality", None) is not None:
        if not isinstance(params["dimensionality"], int):
            raise StyxValidationError(f'`dimensionality` has the wrong type: Received `{type(params.get("dimensionality", None))}` expected `typing.Literal[2, 3] | None`')
        if params["dimensionality"] not in [2, 3]:
            raise StyxValidationError("Parameter `dimensionality` must be one of [2, 3]")
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')
    if params.get("atlas_image_mrf", None) is not None:
        if not isinstance(params["atlas_image_mrf"], str):
            raise StyxValidationError(f'`atlas_image_mrf` has the wrong type: Received `{type(params.get("atlas_image_mrf", None))}` expected `str | None`')
    if params.get("atlas_segmentation_mrf", None) is not None:
        if not isinstance(params["atlas_segmentation_mrf"], str):
            raise StyxValidationError(f'`atlas_segmentation_mrf` has the wrong type: Received `{type(params.get("atlas_segmentation_mrf", None))}` expected `str | None`')
    if params.get("rigid_transform", None) is not None:
        if not isinstance(params["rigid_transform"], str):
            raise StyxValidationError(f'`rigid_transform` has the wrong type: Received `{type(params.get("rigid_transform", None))}` expected `str | None`')
    if params.get("similarity_metric", None) is not None:
        if not isinstance(params["similarity_metric"], str):
            raise StyxValidationError(f'`similarity_metric` has the wrong type: Received `{type(params.get("similarity_metric", None))}` expected `str | None`')
    if params.get("other_options", None) is not None:
        if not isinstance(params["other_options"], str):
            raise StyxValidationError(f'`other_options` has the wrong type: Received `{type(params.get("other_options", None))}` expected `str | None`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')
    if params.get("target_image", None) is None:
        raise StyxValidationError("`target_image` must not be None")
    if not isinstance(params["target_image"], str):
        raise StyxValidationError(f'`target_image` has the wrong type: Received `{type(params.get("target_image", None))}` expected `str`')
    if params.get("mask_image", None) is None:
        raise StyxValidationError("`mask_image` must not be None")
    if not isinstance(params["mask_image"], str):
        raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `str`')
    if params.get("rigid_transform_additional_options", None) is not None:
        if not isinstance(params["rigid_transform_additional_options"], str):
            raise StyxValidationError(f'`rigid_transform_additional_options` has the wrong type: Received `{type(params.get("rigid_transform_additional_options", None))}` expected `str | None`')
    if params.get("similarity_metric_additional_options", None) is not None:
        if not isinstance(params["similarity_metric_additional_options"], str):
            raise StyxValidationError(f'`similarity_metric_additional_options` has the wrong type: Received `{type(params.get("similarity_metric_additional_options", None))}` expected `str | None`')


def ants_joint_label_fusion_sh_cargs(
    params: AntsJointLabelFusionShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsJointLabelFusion.sh")
    if params.get("dimensionality", None) is not None:
        cargs.extend([
            "-d",
            str(params.get("dimensionality", None))
        ])
    if params.get("output", None) is not None:
        cargs.extend([
            "-o",
            params.get("output", None)
        ])
    if params.get("atlas_image_mrf", None) is not None:
        cargs.extend([
            "-a",
            params.get("atlas_image_mrf", None)
        ])
    if params.get("atlas_segmentation_mrf", None) is not None:
        cargs.extend([
            "-l",
            params.get("atlas_segmentation_mrf", None)
        ])
    if params.get("rigid_transform", None) is not None:
        cargs.extend([
            "-g",
            params.get("rigid_transform", None)
        ])
    if params.get("similarity_metric", None) is not None:
        cargs.extend([
            "-s",
            params.get("similarity_metric", None)
        ])
    if params.get("other_options", None) is not None:
        cargs.extend([
            "-k",
            params.get("other_options", None)
        ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "-v",
            ("1" if params.get("verbose", None) else "0")
        ])
    cargs.extend([
        "-i",
        params.get("target_image", None)
    ])
    cargs.extend([
        "-m",
        params.get("mask_image", None)
    ])
    if params.get("rigid_transform_additional_options", None) is not None:
        cargs.extend([
            "-g",
            params.get("rigid_transform_additional_options", None)
        ])
    if params.get("similarity_metric_additional_options", None) is not None:
        cargs.extend([
            "-x",
            params.get("similarity_metric_additional_options", None)
        ])
    return cargs


def ants_joint_label_fusion_sh_outputs(
    params: AntsJointLabelFusionShParamsDict,
    execution: Execution,
) -> AntsJointLabelFusionShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsJointLabelFusionShOutputs(
        root=execution.output_file("."),
        segmentation_output=execution.output_file(params.get("output", None) + ".nii.gz") if (params.get("output") is not None) else None,
    )
    return ret


def ants_joint_label_fusion_sh_execute(
    params: AntsJointLabelFusionShParamsDict,
    runner: Runner | None = None,
) -> AntsJointLabelFusionShOutputs:
    """
    antsJointLabelFusion.sh
    
    The antsJointLabelFusion script is used for performing label fusion using
    multiple atlases to improve segmentation accuracy.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointLabelFusionShOutputs`).
    """
    ants_joint_label_fusion_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_JOINT_LABEL_FUSION_SH_METADATA)
    params = execution.params(params)
    cargs = ants_joint_label_fusion_sh_cargs(params, execution)
    ret = ants_joint_label_fusion_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_joint_label_fusion_sh(
    target_image: str,
    mask_image: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    output: str | None = None,
    atlas_image_mrf: str | None = None,
    atlas_segmentation_mrf: str | None = None,
    rigid_transform: str | None = None,
    similarity_metric: str | None = None,
    other_options: str | None = None,
    verbose: bool | None = None,
    rigid_transform_additional_options: str | None = None,
    similarity_metric_additional_options: str | None = None,
    runner: Runner | None = None,
) -> AntsJointLabelFusionShOutputs:
    """
    antsJointLabelFusion.sh
    
    The antsJointLabelFusion script is used for performing label fusion using
    multiple atlases to improve segmentation accuracy.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        target_image: Image to segment.
        mask_image: Mask image.
        dimensionality: Image dimensionality (2 or 3).
        output: Root directory for the output segmentation.
        atlas_image_mrf: Atlas image(s) to be used for MRF initialization.
        atlas_segmentation_mrf: Atlas segmentation(s) to be used for MRF\
            initialization.
        rigid_transform: Rigid transform initialization.
        similarity_metric: Metric used for calculating similarity.
        other_options: Additional options for label fusion.
        verbose: Verbose output.
        rigid_transform_additional_options: Additional options for rigid\
            transform.
        similarity_metric_additional_options: Additional options for similarity\
            metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointLabelFusionShOutputs`).
    """
    params = ants_joint_label_fusion_sh_params(
        dimensionality=dimensionality,
        output=output,
        atlas_image_mrf=atlas_image_mrf,
        atlas_segmentation_mrf=atlas_segmentation_mrf,
        rigid_transform=rigid_transform,
        similarity_metric=similarity_metric,
        other_options=other_options,
        verbose=verbose,
        target_image=target_image,
        mask_image=mask_image,
        rigid_transform_additional_options=rigid_transform_additional_options,
        similarity_metric_additional_options=similarity_metric_additional_options,
    )
    return ants_joint_label_fusion_sh_execute(params, runner)


__all__ = [
    "ANTS_JOINT_LABEL_FUSION_SH_METADATA",
    "AntsJointLabelFusionShOutputs",
    "AntsJointLabelFusionShParamsDict",
    "AntsJointLabelFusionShParamsDictTagged",
    "ants_joint_label_fusion_sh",
    "ants_joint_label_fusion_sh_execute",
    "ants_joint_label_fusion_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TIME_SCCAN_METADATA = Metadata(
    id="df24e51b7f0bd36e46f31654254028b83351eca4.boutiques",
    name="TimeSCCAN",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_TimeSccanTimeseriesimageToMatrixParamsDictNoTag = typing.TypedDict('_TimeSccanTimeseriesimageToMatrixParamsDictNoTag', {
    "timeseries_image": InputPathType,
    "mask_image": InputPathType,
})
TimeSccanTimeseriesimageToMatrixParamsDictTagged = typing.TypedDict('TimeSccanTimeseriesimageToMatrixParamsDictTagged', {
    "@type": typing.Literal["timeseriesimage_to_matrix"],
    "timeseries_image": InputPathType,
    "mask_image": InputPathType,
})
TimeSccanTimeseriesimageToMatrixParamsDict = _TimeSccanTimeseriesimageToMatrixParamsDictNoTag | TimeSccanTimeseriesimageToMatrixParamsDictTagged


_TimeSccanNetworkSccaParamsDictNoTag = typing.TypedDict('_TimeSccanNetworkSccaParamsDictNoTag', {
    "time_matrix": InputPathType,
    "label_matrix": InputPathType,
})
TimeSccanNetworkSccaParamsDictTagged = typing.TypedDict('TimeSccanNetworkSccaParamsDictTagged', {
    "@type": typing.Literal["network_scca"],
    "time_matrix": InputPathType,
    "label_matrix": InputPathType,
})
TimeSccanNetworkSccaParamsDict = _TimeSccanNetworkSccaParamsDictNoTag | TimeSccanNetworkSccaParamsDictTagged


_TimeSccanNetworkRegionAveragingParamsDictNoTag = typing.TypedDict('_TimeSccanNetworkRegionAveragingParamsDictNoTag', {
    "time_matrix": InputPathType,
    "label_matrix": InputPathType,
})
TimeSccanNetworkRegionAveragingParamsDictTagged = typing.TypedDict('TimeSccanNetworkRegionAveragingParamsDictTagged', {
    "@type": typing.Literal["network_region_averaging"],
    "time_matrix": InputPathType,
    "label_matrix": InputPathType,
})
TimeSccanNetworkRegionAveragingParamsDict = _TimeSccanNetworkRegionAveragingParamsDictNoTag | TimeSccanNetworkRegionAveragingParamsDictTagged


_TimeSccanParamsDictNoTag = typing.TypedDict('_TimeSccanParamsDictNoTag', {
    "output": str,
    "number_consecutive_labels": typing.NotRequired[int | None],
    "minimum_region_size": typing.NotRequired[int | None],
    "iterations": typing.NotRequired[int | None],
    "sparsity": typing.NotRequired[float | None],
    "n_eigenvectors": typing.NotRequired[int | None],
    "robustify": typing.NotRequired[int | None],
    "l1": typing.NotRequired[float | None],
    "cluster_thresh": typing.NotRequired[int | None],
    "ridge_cca": typing.NotRequired[int | None],
    "partial_scca_option": typing.NotRequired[typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None],
    "timeseriesimage_to_matrix": typing.NotRequired[TimeSccanTimeseriesimageToMatrixParamsDict | None],
    "labelsimage_to_matrix": typing.NotRequired[InputPathType | None],
    "network": typing.NotRequired[typing.Union[TimeSccanNetworkSccaParamsDictTagged, TimeSccanNetworkRegionAveragingParamsDictTagged] | None],
})
TimeSccanParamsDictTagged = typing.TypedDict('TimeSccanParamsDictTagged', {
    "@type": typing.Literal["ants/TimeSCCAN"],
    "output": str,
    "number_consecutive_labels": typing.NotRequired[int | None],
    "minimum_region_size": typing.NotRequired[int | None],
    "iterations": typing.NotRequired[int | None],
    "sparsity": typing.NotRequired[float | None],
    "n_eigenvectors": typing.NotRequired[int | None],
    "robustify": typing.NotRequired[int | None],
    "l1": typing.NotRequired[float | None],
    "cluster_thresh": typing.NotRequired[int | None],
    "ridge_cca": typing.NotRequired[int | None],
    "partial_scca_option": typing.NotRequired[typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None],
    "timeseriesimage_to_matrix": typing.NotRequired[TimeSccanTimeseriesimageToMatrixParamsDict | None],
    "labelsimage_to_matrix": typing.NotRequired[InputPathType | None],
    "network": typing.NotRequired[typing.Union[TimeSccanNetworkSccaParamsDictTagged, TimeSccanNetworkRegionAveragingParamsDictTagged] | None],
})
TimeSccanParamsDict = _TimeSccanParamsDictNoTag | TimeSccanParamsDictTagged


def time_sccan_network_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "network_scca": time_sccan_network_scca_cargs,
        "network_region_averaging": time_sccan_network_region_averaging_cargs,
    }.get(t)


def time_sccan_network_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "network_scca": time_sccan_network_scca_validate,
        "network_region_averaging": time_sccan_network_region_averaging_validate,
    }.get(t)


def time_sccan_timeseriesimage_to_matrix(
    timeseries_image: InputPathType,
    mask_image: InputPathType,
) -> TimeSccanTimeseriesimageToMatrixParamsDictTagged:
    """
    Build parameters.
    
    Args:
        timeseries_image:.
        mask_image:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "timeseriesimage_to_matrix",
        "timeseries_image": timeseries_image,
        "mask_image": mask_image,
    }
    return params


def time_sccan_timeseriesimage_to_matrix_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TimeSccanTimeseriesimageToMatrixParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("timeseries_image", None) is None:
        raise StyxValidationError("`timeseries_image` must not be None")
    if not isinstance(params["timeseries_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`timeseries_image` has the wrong type: Received `{type(params.get("timeseries_image", None))}` expected `InputPathType`')
    if params.get("mask_image", None) is None:
        raise StyxValidationError("`mask_image` must not be None")
    if not isinstance(params["mask_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType`')


def time_sccan_timeseriesimage_to_matrix_cargs(
    params: TimeSccanTimeseriesimageToMatrixParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("timeseries_image", None)) + execution.input_file(params.get("mask_image", None)))
    return cargs


def time_sccan_network_scca(
    time_matrix: InputPathType,
    label_matrix: InputPathType,
) -> TimeSccanNetworkSccaParamsDictTagged:
    """
    Build parameters.
    
    Args:
        time_matrix:.
        label_matrix:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "network_scca",
        "time_matrix": time_matrix,
        "label_matrix": label_matrix,
    }
    return params


def time_sccan_network_scca_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TimeSccanNetworkSccaParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("time_matrix", None) is None:
        raise StyxValidationError("`time_matrix` must not be None")
    if not isinstance(params["time_matrix"], (pathlib.Path, str)):
        raise StyxValidationError(f'`time_matrix` has the wrong type: Received `{type(params.get("time_matrix", None))}` expected `InputPathType`')
    if params.get("label_matrix", None) is None:
        raise StyxValidationError("`label_matrix` must not be None")
    if not isinstance(params["label_matrix"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label_matrix` has the wrong type: Received `{type(params.get("label_matrix", None))}` expected `InputPathType`')


def time_sccan_network_scca_cargs(
    params: TimeSccanNetworkSccaParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("scca[" + execution.input_file(params.get("time_matrix", None)) + "," + execution.input_file(params.get("label_matrix", None)) + "]")
    return cargs


def time_sccan_network_region_averaging(
    time_matrix: InputPathType,
    label_matrix: InputPathType,
) -> TimeSccanNetworkRegionAveragingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        time_matrix:.
        label_matrix:.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "network_region_averaging",
        "time_matrix": time_matrix,
        "label_matrix": label_matrix,
    }
    return params


def time_sccan_network_region_averaging_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TimeSccanNetworkRegionAveragingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("time_matrix", None) is None:
        raise StyxValidationError("`time_matrix` must not be None")
    if not isinstance(params["time_matrix"], (pathlib.Path, str)):
        raise StyxValidationError(f'`time_matrix` has the wrong type: Received `{type(params.get("time_matrix", None))}` expected `InputPathType`')
    if params.get("label_matrix", None) is None:
        raise StyxValidationError("`label_matrix` must not be None")
    if not isinstance(params["label_matrix"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label_matrix` has the wrong type: Received `{type(params.get("label_matrix", None))}` expected `InputPathType`')


def time_sccan_network_region_averaging_cargs(
    params: TimeSccanNetworkRegionAveragingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("region-averaging[" + execution.input_file(params.get("time_matrix", None)) + "," + execution.input_file(params.get("label_matrix", None)) + "]")
    return cargs


class TimeSccanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TimeSccanParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    correlation_matrix: OutputPathType
    """The output is the 2D correlation matrix."""


def time_sccan_params(
    output: str,
    number_consecutive_labels: int | None = None,
    minimum_region_size: int | None = None,
    iterations: int | None = None,
    sparsity: float | None = None,
    n_eigenvectors: int | None = None,
    robustify: int | None = None,
    l1: float | None = None,
    cluster_thresh: int | None = None,
    ridge_cca: int | None = None,
    partial_scca_option: typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None = None,
    timeseriesimage_to_matrix: TimeSccanTimeseriesimageToMatrixParamsDict | None = None,
    labelsimage_to_matrix: InputPathType | None = None,
    network: typing.Union[TimeSccanNetworkSccaParamsDictTagged, TimeSccanNetworkRegionAveragingParamsDictTagged] | None = None,
) -> TimeSccanParamsDictTagged:
    """
    Build parameters.
    
    Args:
        output: Output is a 2D correlation matrix.
        number_consecutive_labels: Number of consecutive labels in data.
        minimum_region_size: Minimum size of a region: regions below this size\
            are given a 0.0 connectivity value.
        iterations: Number of iterations.
        sparsity: Sparsity - a float from (0,1] indicating what fraction of the\
            data to use.
        n_eigenvectors: Number of permutations to use in scca.
        robustify: Rank-based scca.
        l1: Use l1 ( > 0 ) or l0 ( < 0 ) penalty, also sets gradient step size\
            e.g. -l 0.5 ( L1 ) , -l -0.5 (L0) will set 0.5 grad descent step for\
            either penalty.
        cluster_thresh: Cluster threshold on view P.
        ridge_cca: Number of permutations to use in scca.
        partial_scca_option: Choices for partial SCCA: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format as output. If the mask has multiple labels\
            (more than one), then the average time series in each label will be\
            computed and put in the csv.
        labelsimage_to_matrix: Takes a labeled (3D) image and converts it to a\
            2D matrix csv format as output.
        network: Build the network connectivity matrix.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/TimeSCCAN",
        "output": output,
    }
    if number_consecutive_labels is not None:
        params["number_consecutive_labels"] = number_consecutive_labels
    if minimum_region_size is not None:
        params["minimum_region_size"] = minimum_region_size
    if iterations is not None:
        params["iterations"] = iterations
    if sparsity is not None:
        params["sparsity"] = sparsity
    if n_eigenvectors is not None:
        params["n_eigenvectors"] = n_eigenvectors
    if robustify is not None:
        params["robustify"] = robustify
    if l1 is not None:
        params["l1"] = l1
    if cluster_thresh is not None:
        params["cluster_thresh"] = cluster_thresh
    if ridge_cca is not None:
        params["ridge_cca"] = ridge_cca
    if partial_scca_option is not None:
        params["partial_scca_option"] = partial_scca_option
    if timeseriesimage_to_matrix is not None:
        params["timeseriesimage_to_matrix"] = timeseriesimage_to_matrix
    if labelsimage_to_matrix is not None:
        params["labelsimage_to_matrix"] = labelsimage_to_matrix
    if network is not None:
        params["network"] = network
    return params


def time_sccan_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TimeSccanParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("number_consecutive_labels", None) is not None:
        if not isinstance(params["number_consecutive_labels"], int):
            raise StyxValidationError(f'`number_consecutive_labels` has the wrong type: Received `{type(params.get("number_consecutive_labels", None))}` expected `int | None`')
    if params.get("minimum_region_size", None) is not None:
        if not isinstance(params["minimum_region_size"], int):
            raise StyxValidationError(f'`minimum_region_size` has the wrong type: Received `{type(params.get("minimum_region_size", None))}` expected `int | None`')
    if params.get("iterations", None) is not None:
        if not isinstance(params["iterations"], int):
            raise StyxValidationError(f'`iterations` has the wrong type: Received `{type(params.get("iterations", None))}` expected `int | None`')
    if params.get("sparsity", None) is not None:
        if not isinstance(params["sparsity"], (float, int)):
            raise StyxValidationError(f'`sparsity` has the wrong type: Received `{type(params.get("sparsity", None))}` expected `float | None`')
    if params.get("n_eigenvectors", None) is not None:
        if not isinstance(params["n_eigenvectors"], int):
            raise StyxValidationError(f'`n_eigenvectors` has the wrong type: Received `{type(params.get("n_eigenvectors", None))}` expected `int | None`')
    if params.get("robustify", None) is not None:
        if not isinstance(params["robustify"], int):
            raise StyxValidationError(f'`robustify` has the wrong type: Received `{type(params.get("robustify", None))}` expected `int | None`')
    if params.get("l1", None) is not None:
        if not isinstance(params["l1"], (float, int)):
            raise StyxValidationError(f'`l1` has the wrong type: Received `{type(params.get("l1", None))}` expected `float | None`')
    if params.get("cluster_thresh", None) is not None:
        if not isinstance(params["cluster_thresh"], int):
            raise StyxValidationError(f'`cluster_thresh` has the wrong type: Received `{type(params.get("cluster_thresh", None))}` expected `int | None`')
    if params.get("ridge_cca", None) is not None:
        if not isinstance(params["ridge_cca"], int):
            raise StyxValidationError(f'`ridge_cca` has the wrong type: Received `{type(params.get("ridge_cca", None))}` expected `int | None`')
    if params.get("partial_scca_option", None) is not None:
        if not isinstance(params["partial_scca_option"], str):
            raise StyxValidationError(f'`partial_scca_option` has the wrong type: Received `{type(params.get("partial_scca_option", None))}` expected `typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None`')
        if params["partial_scca_option"] not in ["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"]:
            raise StyxValidationError("Parameter `partial_scca_option` must be one of [\"PQ\", \"PminusRQ\", \"PQminusR\", \"PminusRQminusR\"]")
    if params.get("timeseriesimage_to_matrix", None) is not None:
        time_sccan_timeseriesimage_to_matrix_validate(params["timeseriesimage_to_matrix"])
    if params.get("labelsimage_to_matrix", None) is not None:
        if not isinstance(params["labelsimage_to_matrix"], (pathlib.Path, str)):
            raise StyxValidationError(f'`labelsimage_to_matrix` has the wrong type: Received `{type(params.get("labelsimage_to_matrix", None))}` expected `InputPathType | None`')
    if params.get("network", None) is not None:
        if not isinstance(params["network"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["network"])}\'')
        if "@type" not in params["network"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["network"]["@type"] not in ["network_scca", "network_region_averaging"]:
            raise StyxValidationError("Parameter `network`s `@type` must be one of [\"network_scca\", \"network_region_averaging\"]")
        time_sccan_network_validate_dyn_fn(params["network"]["@type"])(params["network"])


def time_sccan_cargs(
    params: TimeSccanParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("TimeSCCAN")
    cargs.extend([
        "--output",
        params.get("output", None)
    ])
    if params.get("number_consecutive_labels", None) is not None:
        cargs.extend([
            "-l",
            str(params.get("number_consecutive_labels", None))
        ])
    if params.get("minimum_region_size", None) is not None:
        cargs.extend([
            "-R",
            str(params.get("minimum_region_size", None))
        ])
    if params.get("iterations", None) is not None:
        cargs.extend([
            "-i",
            str(params.get("iterations", None))
        ])
    if params.get("sparsity", None) is not None:
        cargs.extend([
            "-s",
            str(params.get("sparsity", None))
        ])
    if params.get("n_eigenvectors", None) is not None:
        cargs.extend([
            "-n",
            str(params.get("n_eigenvectors", None))
        ])
    if params.get("robustify", None) is not None:
        cargs.extend([
            "-r",
            str(params.get("robustify", None))
        ])
    if params.get("l1", None) is not None:
        cargs.extend([
            "-l",
            str(params.get("l1", None))
        ])
    if params.get("cluster_thresh", None) is not None:
        cargs.extend([
            "--ClusterThresh",
            str(params.get("cluster_thresh", None))
        ])
    if params.get("ridge_cca", None) is not None:
        cargs.extend([
            "-e",
            str(params.get("ridge_cca", None))
        ])
    if params.get("partial_scca_option", None) is not None:
        cargs.extend([
            "--partial-scca-option",
            params.get("partial_scca_option", None)
        ])
    if params.get("timeseriesimage_to_matrix", None) is not None:
        cargs.extend([
            "--timeseriesimage-to-matrix",
            *time_sccan_timeseriesimage_to_matrix_cargs(params.get("timeseriesimage_to_matrix", None), execution)
        ])
    if params.get("labelsimage_to_matrix", None) is not None:
        cargs.extend([
            "--labelsimage-to-matrix",
            execution.input_file(params.get("labelsimage_to_matrix", None))
        ])
    if params.get("network", None) is not None:
        cargs.extend([
            "--network",
            *time_sccan_network_cargs_dyn_fn(params.get("network", None)["@type"])(params.get("network", None), execution)
        ])
    return cargs


def time_sccan_outputs(
    params: TimeSccanParamsDict,
    execution: Execution,
) -> TimeSccanOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TimeSccanOutputs(
        root=execution.output_file("."),
        correlation_matrix=execution.output_file(params.get("output", None)),
    )
    return ret


def time_sccan_execute(
    params: TimeSccanParamsDict,
    runner: Runner | None = None,
) -> TimeSccanOutputs:
    """
    TimeSCCAN
    
    A tool for sparse statistical analysis on connectivity within a subject.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TimeSccanOutputs`).
    """
    time_sccan_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TIME_SCCAN_METADATA)
    params = execution.params(params)
    cargs = time_sccan_cargs(params, execution)
    ret = time_sccan_outputs(params, execution)
    execution.run(cargs)
    return ret


def time_sccan(
    output: str,
    number_consecutive_labels: int | None = None,
    minimum_region_size: int | None = None,
    iterations: int | None = None,
    sparsity: float | None = None,
    n_eigenvectors: int | None = None,
    robustify: int | None = None,
    l1: float | None = None,
    cluster_thresh: int | None = None,
    ridge_cca: int | None = None,
    partial_scca_option: typing.Literal["PQ", "PminusRQ", "PQminusR", "PminusRQminusR"] | None = None,
    timeseriesimage_to_matrix: TimeSccanTimeseriesimageToMatrixParamsDict | None = None,
    labelsimage_to_matrix: InputPathType | None = None,
    network: typing.Union[TimeSccanNetworkSccaParamsDictTagged, TimeSccanNetworkRegionAveragingParamsDictTagged] | None = None,
    runner: Runner | None = None,
) -> TimeSccanOutputs:
    """
    TimeSCCAN
    
    A tool for sparse statistical analysis on connectivity within a subject.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        output: Output is a 2D correlation matrix.
        number_consecutive_labels: Number of consecutive labels in data.
        minimum_region_size: Minimum size of a region: regions below this size\
            are given a 0.0 connectivity value.
        iterations: Number of iterations.
        sparsity: Sparsity - a float from (0,1] indicating what fraction of the\
            data to use.
        n_eigenvectors: Number of permutations to use in scca.
        robustify: Rank-based scca.
        l1: Use l1 ( > 0 ) or l0 ( < 0 ) penalty, also sets gradient step size\
            e.g. -l 0.5 ( L1 ) , -l -0.5 (L0) will set 0.5 grad descent step for\
            either penalty.
        cluster_thresh: Cluster threshold on view P.
        ridge_cca: Number of permutations to use in scca.
        partial_scca_option: Choices for partial SCCA: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format as output. If the mask has multiple labels\
            (more than one), then the average time series in each label will be\
            computed and put in the csv.
        labelsimage_to_matrix: Takes a labeled (3D) image and converts it to a\
            2D matrix csv format as output.
        network: Build the network connectivity matrix.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TimeSccanOutputs`).
    """
    params = time_sccan_params(
        output=output,
        number_consecutive_labels=number_consecutive_labels,
        minimum_region_size=minimum_region_size,
        iterations=iterations,
        sparsity=sparsity,
        n_eigenvectors=n_eigenvectors,
        robustify=robustify,
        l1=l1,
        cluster_thresh=cluster_thresh,
        ridge_cca=ridge_cca,
        partial_scca_option=partial_scca_option,
        timeseriesimage_to_matrix=timeseriesimage_to_matrix,
        labelsimage_to_matrix=labelsimage_to_matrix,
        network=network,
    )
    return time_sccan_execute(params, runner)


__all__ = [
    "TIME_SCCAN_METADATA",
    "TimeSccanNetworkRegionAveragingParamsDict",
    "TimeSccanNetworkRegionAveragingParamsDictTagged",
    "TimeSccanNetworkSccaParamsDict",
    "TimeSccanNetworkSccaParamsDictTagged",
    "TimeSccanOutputs",
    "TimeSccanParamsDict",
    "TimeSccanParamsDictTagged",
    "TimeSccanTimeseriesimageToMatrixParamsDict",
    "TimeSccanTimeseriesimageToMatrixParamsDictTagged",
    "time_sccan",
    "time_sccan_execute",
    "time_sccan_network_region_averaging",
    "time_sccan_network_scca",
    "time_sccan_params",
    "time_sccan_timeseriesimage_to_matrix",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_ALIGN_ORIGIN_METADATA = Metadata(
    id="a2901ef9bcab2f861ffe659998f028f791c5e4fd.boutiques",
    name="antsAlignOrigin",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsAlignOriginParamsDictNoTag = typing.TypedDict('_AntsAlignOriginParamsDictNoTag', {
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "input": InputPathType,
    "reference_image": InputPathType,
    "output": str,
})
AntsAlignOriginParamsDictTagged = typing.TypedDict('AntsAlignOriginParamsDictTagged', {
    "@type": typing.Literal["ants/antsAlignOrigin"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3] | None],
    "input": InputPathType,
    "reference_image": InputPathType,
    "output": str,
})
AntsAlignOriginParamsDict = _AntsAlignOriginParamsDictNoTag | AntsAlignOriginParamsDictTagged


class AntsAlignOriginOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsAlignOriginParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    aligned_image: OutputPathType
    """The output is the aligned image."""


def ants_align_origin_params(
    input_: InputPathType,
    reference_image: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
) -> AntsAlignOriginParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: Currently, the only input objects supported are image objects.\
            However, the current framework allows for warping of other objects such\
            as meshes and point sets.
        reference_image: For warping input images, the reference image defines\
            the spacing, origin, size, and direction of the output warped image.
        output: One can either output the warped image or, if the boolean is\
            set, one can print out the displacement field based on the composite\
            transform and the reference image.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, antsWarp tries to infer\
            the dimensionality from the input image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsAlignOrigin",
        "input": input_,
        "reference_image": reference_image,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    return params


def ants_align_origin_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsAlignOriginParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensionality", None) is not None:
        if not isinstance(params["dimensionality"], int):
            raise StyxValidationError(f'`dimensionality` has the wrong type: Received `{type(params.get("dimensionality", None))}` expected `typing.Literal[2, 3] | None`')
        if params["dimensionality"] not in [2, 3]:
            raise StyxValidationError("Parameter `dimensionality` must be one of [2, 3]")
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("reference_image", None) is None:
        raise StyxValidationError("`reference_image` must not be None")
    if not isinstance(params["reference_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`reference_image` has the wrong type: Received `{type(params.get("reference_image", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def ants_align_origin_cargs(
    params: AntsAlignOriginParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsAlignOrigin")
    if params.get("dimensionality", None) is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality", None))
        ])
    cargs.extend([
        "--input",
        execution.input_file(params.get("input", None))
    ])
    cargs.extend([
        "--reference-image",
        execution.input_file(params.get("reference_image", None))
    ])
    cargs.extend([
        "--output",
        params.get("output", None)
    ])
    return cargs


def ants_align_origin_outputs(
    params: AntsAlignOriginParamsDict,
    execution: Execution,
) -> AntsAlignOriginOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsAlignOriginOutputs(
        root=execution.output_file("."),
        aligned_image=execution.output_file(params.get("output", None)),
    )
    return ret


def ants_align_origin_execute(
    params: AntsAlignOriginParamsDict,
    runner: Runner | None = None,
) -> AntsAlignOriginOutputs:
    """
    antsAlignOrigin
    
    antsAlignOrigin, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAlignOriginOutputs`).
    """
    ants_align_origin_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_ALIGN_ORIGIN_METADATA)
    params = execution.params(params)
    cargs = ants_align_origin_cargs(params, execution)
    ret = ants_align_origin_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_align_origin(
    input_: InputPathType,
    reference_image: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    runner: Runner | None = None,
) -> AntsAlignOriginOutputs:
    """
    antsAlignOrigin
    
    antsAlignOrigin, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_: Currently, the only input objects supported are image objects.\
            However, the current framework allows for warping of other objects such\
            as meshes and point sets.
        reference_image: For warping input images, the reference image defines\
            the spacing, origin, size, and direction of the output warped image.
        output: One can either output the warped image or, if the boolean is\
            set, one can print out the displacement field based on the composite\
            transform and the reference image.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, antsWarp tries to infer\
            the dimensionality from the input image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAlignOriginOutputs`).
    """
    params = ants_align_origin_params(
        dimensionality=dimensionality,
        input_=input_,
        reference_image=reference_image,
        output=output,
    )
    return ants_align_origin_execute(params, runner)


__all__ = [
    "ANTS_ALIGN_ORIGIN_METADATA",
    "AntsAlignOriginOutputs",
    "AntsAlignOriginParamsDict",
    "AntsAlignOriginParamsDictTagged",
    "ants_align_origin",
    "ants_align_origin_execute",
    "ants_align_origin_params",
]
